package com.qmth.teachcloud.common.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.common.entity.MarkStudent;
import com.qmth.teachcloud.common.enums.mark.SubjectiveStatus;
import com.qmth.teachcloud.common.mapper.MarkStudentMapper;
import com.qmth.teachcloud.common.service.MarkStudentService;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 考试考生库 服务实现类
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@Service
public class MarkStudentServiceImpl extends ServiceImpl<MarkStudentMapper, MarkStudent> implements MarkStudentService {

    @Override
    public List<String> listClassByExamIdAndCourseCode(Long examId, String paperNumber) {
        QueryWrapper<MarkStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkStudent::getExamId, examId)
                .eq(MarkStudent::getPaperNumber, paperNumber);
        List<MarkStudent> markStudentList = this.list(queryWrapper);

        List<String> classNameList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(markStudentList)) {
            classNameList = markStudentList.stream().filter(m -> StringUtils.isNotBlank(m.getClassName())).map(MarkStudent::getClassName).distinct().collect(Collectors.toList());
        }
        return classNameList;
    }

    @Override
    public void updateSubjectiveStatusAndScore(Long studentId, SubjectiveStatus status, int score, String scoreList) {
        UpdateWrapper<MarkStudent> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkStudent::getSubjectiveStatus, status)
                .set(MarkStudent::getSubjectiveScore, score)
                .set(MarkStudent::getSubjectiveScoreList, scoreList)
                .eq(MarkStudent::getId, studentId);
        this.update(updateWrapper);
    }
}
