package com.qmth.teachcloud.mark.service.impl;

import com.qmth.teachcloud.common.entity.MarkPaper;
import com.qmth.teachcloud.common.enums.mark.MarkPaperStatus;
import com.qmth.teachcloud.common.enums.mark.MarkProblemStatus;
import com.qmth.teachcloud.common.enums.mark.SubjectiveStatus;
import com.qmth.teachcloud.common.service.MarkPaperService;
import com.qmth.teachcloud.common.service.MarkStudentService;
import com.qmth.teachcloud.mark.entity.MarkGroup;
import com.qmth.teachcloud.mark.entity.MarkGroupStudent;
import com.qmth.teachcloud.mark.entity.MarkTask;
import com.qmth.teachcloud.mark.entity.MarkUserGroup;
import com.qmth.teachcloud.mark.enums.LockType;
import com.qmth.teachcloud.mark.enums.MarkTaskStatus;
import com.qmth.teachcloud.mark.lock.LockService;
import com.qmth.teachcloud.mark.service.*;
import com.qmth.teachcloud.mark.utils.TaskLock;
import com.qmth.teachcloud.mark.utils.TaskLockUtil;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

@Service
public class MarkServiceImpl implements MarkService {

    public static final int UN_SELECTIVE_SCORE = -1;

    private Map<Long, Long> markerLastUpdateTime = new ConcurrentHashMap<>();

    @Resource
    private MarkPaperService markPaperService;
    @Resource
    private MarkGroupService markGroupService;
    @Resource
    private MarkUserGroupService markUserGroupService;
    @Resource
    private MarkTaskService markTaskService;
    @Resource
    private MarkTrackService markTrackService;
    @Resource
    private MarkStudentService markStudentService;
    @Resource
    private MarkGroupStudentService markGroupStudentService;
    @Resource
    private MarkProblemHistoryService markProblemHistoryService;
    @Resource
    LockService lockService;

    /**
     * 某个评卷分组已申请的评卷任务数量
     *
     * @return
     */
    @Override
    public int applyCurrentCount(MarkGroup markGroup) {
        // todo 正在评卷数量查询 (评卷员和评卷分组分开2个方法)
        TaskLock taskLock = getTaskLock(markGroup);
        int count = 0;
        if (taskLock != null) {
            count = taskLock.count();
        }
        return count;
    }

    private TaskLock getTaskLock(MarkGroup markGroup) {
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(markGroup.getExamId(), markGroup.getPaperNumber());
        if (markPaper.getStatus() == MarkPaperStatus.FORMAL) {
            return TaskLockUtil.getFormalTask(getGroupKey(markGroup));
        }
        return null;
    }

    /**
     * 释放某个评卷员的所有锁定任务
     *
     * @param markUserGroup
     */
    @Override
    public void releaseByMarkUserGroup(MarkUserGroup markUserGroup) {
        TaskLock taskLock = TaskLockUtil.getFormalTask(getGroupKey(markUserGroup));
        taskLock.clear(markUserGroup.getId());
    }

    @Override
    public int applyCurrentCount(MarkGroup markGroup, Long markUserGroupId) {
        int count = 0;
        TaskLock taskLock = getTaskLock(markGroup);
        if (taskLock != null) {
            count = taskLock.count(markUserGroupId);
        }
        return count;
    }

    @Override
    public void resetMarker(MarkUserGroup markUserGroup) {
        Long examId = markUserGroup.getExamId();
        String paperNumber = markUserGroup.getPaperNumber();
        Integer groupNumber = markUserGroup.getGroupNumber();
        Long userId = markUserGroup.getUserId();
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(examId, paperNumber);
        if (markPaper == null || markUserGroupService.getById(markUserGroup.getId()) == null) {
            return;
        }
        if (markPaper.getStatus() == MarkPaperStatus.FORMAL) {
            // 遍历相关评卷任务的模式
            List<MarkTaskStatus> statusList = Arrays.asList(MarkTaskStatus.ARBITRATED, MarkTaskStatus.WAIT_ARBITRATE, MarkTaskStatus.PROBLEM);
            List<MarkTask> markTaskList = markTaskService.listByExamIdAndPaperNumberAndGroupNumberAndUserIdAndStatusNotIn(examId, paperNumber, groupNumber, userId, statusList);
            for (MarkTask markTask : markTaskList) {
                Long studentId = markTask.getStudentId();
                markTrackService.deleteByTaskId(markTask.getId());
//                specialTagDao.deleteByLibraryId(library.getId());
                markTaskService.resetById(markTask.getId(), null, null, null, MarkTaskStatus.WAITING);
                lockService.waitlock(LockType.STUDENT, markTask.getStudentId());
                updateStudentGroupStatus(studentId, examId, paperNumber, groupNumber, SubjectiveStatus.UNMARK);
                markStudentService.updateSubjectiveStatusAndScore(studentId, SubjectiveStatus.UNMARK, 0, null);
//                inspectedService.cancelByStudent(library.getStudentId());
                lockService.unlock(LockType.STUDENT, markTask.getStudentId());
            }
            markUserGroupService.resetById(markUserGroup.getId());
        }
        this.updateMarkedCount(examId, paperNumber, groupNumber);
        releaseByMarkUserGroup(markUserGroup);
//        inspectHistoryService.deleteByExamIdAndSubjectCode(marker.getExamId(), marker.getSubjectCode());
    }

    @Override
    public void updateMarkedCount(Long examId, String paperNumber, Integer groupNumber) {
        List<MarkTaskStatus> markTaskStatuses = Arrays.asList(MarkTaskStatus.MARKED, MarkTaskStatus.ARBITRATED);
        int count = markTaskService.countByExamIdAndPaperNumberAndGroupNumberAndStatusIn(examId, paperNumber, groupNumber, markTaskStatuses);
        markGroupService.updateMarkedCount(examId, paperNumber, groupNumber, count);
    }

    @Override
    public boolean rejectMarkTask(MarkTask markTask, Long userId) {
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(markTask.getExamId(), markTask.getPaperNumber());
        if (markPaper.getStatus() == MarkPaperStatus.FINISH) {
            return false;
        }
        Long now = System.currentTimeMillis();
        if (markTaskService.resetById(markTask.getId(), null, userId, now, MarkTaskStatus.WAITING)) {
            markTrackService.deleteByTaskId(markTask.getId());
            resetStudentGroup(markTask.getStudentId(), markTask.getExamId(), markTask.getPaperNumber(), markTask.getGroupNumber());
            markProblemHistoryService.resetByMarkTaskId(markTask.getId(), MarkProblemStatus.WAITING, userId, MarkProblemStatus.BACK, now);
            updateMarkedCount(markTask.getExamId(), markTask.getPaperNumber(), markTask.getGroupNumber());
            return true;
        } else {
            return false;
        }
    }

    private void resetStudentGroup(Long studentId, Long examId, String paperNumber, Integer groupNumber) {
        updateStudentGroupStatus(studentId, examId, paperNumber, groupNumber, SubjectiveStatus.UNMARK);
        markStudentService.updateSubjectiveStatusAndScore(studentId, SubjectiveStatus.UNMARK, 0, null);
    }

    private void updateStudentGroupStatus(Long studentId, Long examId, String paperNumber, Integer groupNumber, SubjectiveStatus status) {
        MarkGroupStudent gs = new MarkGroupStudent();
        gs.setStudentId(studentId);
        gs.setExamId(examId);
        gs.setPaperNumber(paperNumber);
        gs.setGroupNumber(groupNumber);
        gs.setStatus(status);
        markGroupStudentService.save(gs);
    }

    @Override
    public String getGroupKey(MarkGroup markGroup) {
        return markGroup.getExamId() + "_" + markGroup.getPaperNumber() + "_" + markGroup.getNumber();
    }

    @Override
    public void updateQuality(MarkUserGroup markUserGroup) {
        List<MarkTask> list = markTaskService.listByExamIdAndPaperNumberAndGroupNumberAndUserId(markUserGroup.getExamId(), markUserGroup.getPaperNumber(), markUserGroup.getGroupNumber(), markUserGroup.getUserId());
        int finishCount = 0;
        int validCount = 0;
        double sumScore = 0;
        double sumScore2 = 0;
        double avgScore = 0;
        double stdevScore = 0;
        double sumSpent = 0;
        double avgSpent = 0;
        for (MarkTask markTask : list) {
            finishCount++;
            if (markTask.getStatus() == MarkTaskStatus.MARKED) {
                validCount++;
            }
            double score = markTask.getMarkerScore() != null && markTask.getMarkerScore() != UN_SELECTIVE_SCORE ? markTask.getMarkerScore() : 0;
            int spent = markTask.getMarkerSpent() != null ? markTask.getMarkerSpent() : 0;

            sumScore += score;
            sumScore2 += Math.pow(score, 2);
            sumSpent += spent;
            if (finishCount > 0) {
                avgScore = sumScore / finishCount;
                avgSpent = sumSpent / finishCount;
                // 递归法计算标准差
                stdevScore = Math.sqrt(sumScore2 / finishCount - Math.pow(sumScore / finishCount, 2));
            }
        }
        markUserGroupService.updateQualityById(markUserGroup.getId(), finishCount, validCount, avgSpent / 1000, avgScore, stdevScore);
        markerLastUpdateTime.put(markUserGroup.getId(), System.currentTimeMillis());
    }

    @Override
    public boolean needUpdateQuality(MarkUserGroup markUserGroup, int expireMinutes) {
        if (markUserGroup == null) {
            return false;
        }
        Long lastUpdateTime = markerLastUpdateTime.get(markUserGroup.getId());
        MarkTask markTask = markTaskService.getLastOneByUserIdAndStatus(markUserGroup.getExamId(), markUserGroup.getPaperNumber(), markUserGroup.getGroupNumber(), markUserGroup.getUserId(), MarkTaskStatus.MARKED);
        if (markTask != null && markTask.getMarkerTime() != null) {
            long lastMarkTime = markTask.getMarkerTime();
            if (lastUpdateTime != null && lastUpdateTime > lastMarkTime) {
                return false;
            } else {
                return (System.currentTimeMillis() - lastMarkTime) < (expireMinutes * 60 * 1000);
            }
        }
        return false;
    }

    private String getGroupKey(MarkUserGroup markUserGroup) {
        return markUserGroup.getExamId() + "_" + markUserGroup.getPaperNumber() + "_" + markUserGroup.getGroupNumber();
    }
}
